@@ -70,9 +70,11 @@ static xf86ConfigSymTabRec FilesTab[] =
 	{ENDSECTION, "endsection"},
 	{FONTPATH, "fontpath"},
 	{MODULEPATH, "modulepath"},
-	{INPUTDEVICES, "inputdevices"},
 	{LOGFILEPATH, "logfile"},
 	{XKBDIR, "xkbdir"},
+	/* Obsolete keywords that aren't used but shouldn't cause errors: */
+	{OBSOLETE_TOKEN, "rgbpath"},
+	{OBSOLETE_TOKEN, "inputdevices"},
 	{-1, ""},
 };
 
@@ -101,7 +103,7 @@ xf86parseFilesSection (void)
 			str = val.str;
 			if (ptr->file_fontpath == NULL)
 			{
-				ptr->file_fontpath = xf86confmalloc (1);
+				ptr->file_fontpath = malloc (1);
 				ptr->file_fontpath[0] = '\0';
 				i = strlen (str) + 1;
 			}
@@ -115,12 +117,12 @@ xf86parseFilesSection (void)
 				}
 			}
 			ptr->file_fontpath =
-				xf86confrealloc (ptr->file_fontpath, i);
+				realloc (ptr->file_fontpath, i);
 			if (j)
 				strcat (ptr->file_fontpath, ",");
 
 			strcat (ptr->file_fontpath, str);
-			xf86conffree (val.str);
+			free (val.str);
 			break;
 		case MODULEPATH:
 			if (xf86getSubToken (&(ptr->file_comment)) != STRING)
@@ -129,7 +131,7 @@ xf86parseFilesSection (void)
 			str = val.str;
 			if (ptr->file_modulepath == NULL)
 			{
-				ptr->file_modulepath = xf86confmalloc (1);
+				ptr->file_modulepath = malloc (1);
 				ptr->file_modulepath[0] = '\0';
 				k = strlen (str) + 1;
 			}
@@ -142,39 +144,12 @@ xf86parseFilesSection (void)
 					l = TRUE;
 				}
 			}
-			ptr->file_modulepath = xf86confrealloc (ptr->file_modulepath, k);
+			ptr->file_modulepath = realloc (ptr->file_modulepath, k);
 			if (l)
 				strcat (ptr->file_modulepath, ",");
 
 			strcat (ptr->file_modulepath, str);
-			xf86conffree (val.str);
-			break;
-		case INPUTDEVICES:
-			if (xf86getSubToken (&(ptr->file_comment)) != STRING)
-				Error (QUOTE_MSG, "InputDevices");
-			l = FALSE;
-			str = val.str;
-			if (ptr->file_inputdevs == NULL)
-			{
-				ptr->file_inputdevs = xf86confmalloc (1);
-				ptr->file_inputdevs[0] = '\0';
-				k = strlen (str) + 1;
-			}
-			else
-			{
-				k = strlen (ptr->file_inputdevs) + strlen (str) + 1;
-				if (ptr->file_inputdevs[strlen (ptr->file_inputdevs) - 1] != ',')
-				{
-					k++;
-					l = TRUE;
-				}
-			}
-			ptr->file_inputdevs = xf86confrealloc (ptr->file_inputdevs, k);
-			if (l)
-				strcat (ptr->file_inputdevs, ",");
-
-			strcat (ptr->file_inputdevs, str);
-			xf86conffree (val.str);
+			free (val.str);
 			break;
 		case LOGFILEPATH:
 			if (xf86getSubToken (&(ptr->file_comment)) != STRING)
@@ -189,6 +164,10 @@ xf86parseFilesSection (void)
 		case EOF_TOKEN:
 			Error (UNEXPECTED_EOF_MSG, NULL);
 			break;
+		case OBSOLETE_TOKEN:
+			xf86parseError (OBSOLETE_MSG, xf86tokenString ());
+			xf86getSubToken (&(ptr->file_comment));
+			break;
 		default:
 			Error (INVALID_KEYWORD_MSG, xf86tokenString ());
 			break;
@@ -231,21 +210,6 @@ xf86printFileSection (FILE * cf, XF86ConfFilesPtr ptr)
 		}
 		fprintf (cf, "\tModulePath   \"%s\"\n", s);
 	}
-	if (ptr->file_inputdevs)
-	{
-		s = ptr->file_inputdevs;
-		p = index (s, ',');
-		while (p)
-		{
-			*p = '\000';
-			fprintf (cf, "\tInputDevices   \"%s\"\n", s);
-			*p = ',';
-			s = p;
-			s++;
-			p = index (s, ',');
-		}
-		fprintf (cf, "\tInputDevices   \"%s\"\n", s);
-	}
 	if (ptr->file_fontpath)
 	{
 		s = ptr->file_fontpath;
@@ -273,10 +237,9 @@ xf86freeFiles (XF86ConfFilesPtr p)
 
 	TestFree (p->file_logfile);
 	TestFree (p->file_modulepath);
-	TestFree (p->file_inputdevs);
 	TestFree (p->file_fontpath);
 	TestFree (p->file_comment);
 	TestFree (p->file_xkbdir);
 
-	xf86conffree (p);
+	free (p);
 }
